/**
 * commands/dcc.cc
 *
 * (C) 2006-2007 Murat Deligonul
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include "autoconf.h"

#include <list>
#include <algorithm>
#include <functional>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <cctype>
#include <unistd.h>
#include <sys/time.h>
#include "util/strings.h"
#include "logging/chatlog.h"
#include "commands.h"
#include "textline.h"
#include "proxy.h"
#include "dcc.h"
#include "dcc_offer.h"
#include "conn.h"
#include "messages.h"
#include "directory.h"
#include "filelib.h"
#include "file_entry.h"
#include "user.h"
#include "debug.h"

using std::list;
using std::string;
using std::vector;
using namespace util::strings;

/** 
 * DCC Command:
 */
CMDFUNC(dcc)
{
	enum {
		DCC_CMD_LIST = 1,
		DCC_CMD_HELP,
		DCC_CMD_KILL,
		DCC_CMD_INFO,
		DCC_CMD_STORE,
		DCC_CMD_PROXY,
		DCC_CMD_ACCEPT,
	};
	static const struct simple_table cmds[] = {
		{ "list",   DCC_CMD_LIST },
		{ "proxy",  DCC_CMD_PROXY },
		{ "store",  DCC_CMD_STORE },
		{ "info",   DCC_CMD_INFO },
		{ "kill",   DCC_CMD_KILL },
		{ "close",  DCC_CMD_KILL },
		{ "reject", DCC_CMD_KILL },
		{ "get",    DCC_CMD_ACCEPT},
		{ "accept", DCC_CMD_ACCEPT},
		{ "relay",  DCC_CMD_ACCEPT},
		{ "send",   DCC_CMD_ACCEPT},
		{ "help",   DCC_CMD_HELP},
	};

	if (args == 0) {
		cprintf("DCC: Try `/quote ezb help dcc'\r\n");
		return 1;
	}

	const std::list<dcc *>& dcc_list = proxy->dcc_list();
	const std::list<dcc_offer *>& dcc_offer_list = proxy->dcc_offer_list();

	std::list<dcc *>::const_iterator dcc_i = dcc_list.begin(),
					dcc_end = dcc_list.end();
	std::list<dcc_offer *>::const_iterator offer_i = dcc_offer_list.begin(),
						offer_end = dcc_offer_list.end();

	dcc * d = NULL;
	dcc_offer * offer = NULL;
	std::string err;

	int cmd = simple_lookup(args[0], (const simple_table *) &cmds,
			sizeof(cmds) / sizeof(struct simple_table));
	int found = 0;

	if (cmd >= DCC_CMD_KILL) {
		if (args < 2) {
			cprintf("DCC: Usage: DCC %s <id>\r\n", args[0]);
			return 1;
		}
		if (!dcc::lookup_by_id(this, atoi(args[1]), &d, &offer)) {
			cprintf("DCC: ERROR: Invalid id `%s'\r\n", args[1]);
			return 1;
		}
	}
	switch (cmd)
	{
	/** list active dccs and offers */
	case DCC_CMD_LIST:
		cprintf("Active DCC \002Sessions\002 for %s\r\n", addr());
		cprintf("\002ID\002    \002Info\002                                     \002Age\002\r\n");
		cprintf("-------------------------------------------------------------------------\r\n");
		for (; dcc_i != dcc_end; ++dcc_i) {
			d = *dcc_i;
			char infobuff[40] = "[Unknown]";
			if (d->owner() != this || d->status_check(0,0) < 0) {
				continue;
			}
			d->info(infobuff, sizeof(infobuff));
			cprintf("%-5d %-40s %d secs\r\n", d->id(), infobuff, d->age());
			++found;
		}
		if (!found)
			cprintf("-- No active DCC sessions found for you --\r\n");
		found = 0;
		/* print the offers now */
		cprintf("-\r\n");

		if (!is_allowed(user_perms::ENABLE_DCC_FILTER))	{
			cprintf("[DCC Offer filtering is \002off\002]\r\n");
			return 1;
		}

		cprintf("Active DCC \002Offers\002 for %s\r\n", addr());
		cprintf("\002ID\002    \002Info\002                                     \002Age\002\r\n");
		cprintf("-------------------------------------------------------------------------\r\n");
		for (; offer_i != offer_end; ++offer_i) {
			offer = *offer_i;
			char infobuff[40] = "[Unknown]";
			if (offer->owner() != this || offer->status_check(0,0) < 0) {
				continue;
			}

			offer->info(infobuff, sizeof(infobuff));
			cprintf("%-5d %-40s %d secs\r\n", offer->id(), infobuff, offer->age());
			++found;
		}

		if (!found) {
			cprintf("-- No active DCC offers found for you --\r\n");
		}
		break;

		/** store dcc send locally **/
	case DCC_CMD_STORE:
	{
#warning "FIXME: issues with DCC storage and file library access"
		if (!offer) {
			cprintf("DCC: I can only store DCC \002OFFERS\002\r\n");
			return 1;
		} 
		if (offer->status_check(0,0) < 0) {
			cprintf("DCC: Invalid offer\r\n");
			return 1;
		}
		if (!is_allowed(user_perms::ENABLE_DCC_STORAGE)) {
			cprintf("DCC: Sorry, you are not allowed to store files.\r\n");
			return 1;
		}
		if (strcasecmp(offer->type(), "CHAT") == 0) {
			cprintf("DCC: I can't store a chat!\r\n");
			return 1;
		}

		dccget * get = receive_dcc_offer(offer, err);
		if (!get) {
			cprintf("DCC: Error: couldn't receive file '%s' (error: %s)\r\n",
					offer->filename(), err.c_str());
			return 1;
		}
		offer->die(0, NULL);
		// new dcc object automatically added to a global list
		break;
	}
	case DCC_CMD_PROXY:
	{
		if (!offer) {
			cprintf("DCC: I can only proxy DCC \002OFFERS\002\r\n");
			return 1;
		}
		if (((offer->flags() & dcc_offer::INCOMING) &&
					!is_allowed(user_perms::ENABLE_INCOMING_DCC_PROXYING))
				|| ((offer->flags() & dcc_offer::OUTGOING) &&
					!is_allowed(user_perms::ENABLE_OUTGOING_DCC_PROXYING))) {
			cprintf("DCC: Sorry, you are not allowed to use DCC proxying\r\n");
			return 1;
		}
		if (strcasecmp(offer->to(), "(ezbounce)") == 0) {
			cprintf("DCC: I'm not going to proxy a dcc to myself!\r\n");
			return 1;
		}
		if ((offer->flags() & dcc_offer::OUTGOING) && !is_bounced()) {
			cprintf("DCC: You must be connected to IRC to proxy outgoing DCCs\r\n");
			return 1;
		}

		dccpipe * dp = proxy_dcc_offer(offer, err);
		if (!dp) {
			cprintf("DCC: ERROR: couldn't setup proxying (error: %s)\r\n", err.c_str());
			return 1;
		}
		offer->die(0, 0);
		// new dccproxy session added to list
		break;
	}
	case DCC_CMD_KILL:
		if (offer) {
			offer->die(dcc::DCC_USER_KILL, "");
		} 
		else {
			d->die(dcc::DCC_USER_KILL, "");
		}
		cprintf("DCC: Killed dcc/offer #%s\r\n",args[1]);
		break;

	case DCC_CMD_ACCEPT:
		/* relay, accept, etc */
		if (!offer) {
			cprintf("DCC: I can only do this on DCC \002OFFERS\002\r\n");
			return 1;
		}
		if (strcasecmp(offer->to(), "(ezbounce)") == 0) {
			cprintf("DCC: Use `/quote ezb dcc store %s' if you want me to store a file\r\n", args[1]);
			return 1;
		}
		if ((offer->flags() & dcc_offer::OUTGOING) && !is_bounced()) {
			cprintf("DCC: This is an outgoing offer and you are not connected to an IRC server\r\n");
			return 1;
		}

		if (relay_dcc_offer(offer, offer->to(), err)) {
			cprintf("DCC: ERROR: couldn't relay offer (error: %s)\r\n", err.c_str());
			return 1;
		}
		cprintf("DCC: Relayed %s offer %d from %s to %s\r\n",
			offer->type(), offer->id(), offer->from(), offer->to());
		offer->die(0, 0);
		break;

	case DCC_CMD_INFO:
		cprintf("\002Information\002 for DCC/Offer %s\r\n", args[1]);
		if (offer) {
			offer->info(this);
		} 
		else {
			d->info(this);
		}
		break;

	case DCC_CMD_HELP:
		cprintf_multiline(__HELP_ENTRY(dcc));
		break;

	default:
		cprintf("DCC -- Unknown command '%s'\r\n", args[0]);
		break;
	}
	return 1;
}

